#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <locale.h>
#include "config.h"
#include "gcryfolder-password-prompt.h"
#include "gtk-compat.c"

static void
update_button (GObject    *object,
               GParamSpec *pspec,
               gpointer    user_data)
{
  const char *pw1;
  const char *pw2;

  pw1 = gtk_editable_get_text (GTK_EDITABLE (data->password_entry1));
  if (data->new_folder) {
    pw2 = gtk_editable_get_text (GTK_EDITABLE (data->password_entry2));
    gtk_widget_set_sensitive (data->ok_button, pw1[0] != '\0' && g_str_equal (pw1, pw2));
  } else {
    gtk_widget_set_sensitive (data->ok_button, pw1[0] != '\0');
  }
}


static void
ok_button_pressed (GtkButton *widget,
                   GtkWidget *window)
{
  g_print("%s\n", gtk_editable_get_text (GTK_EDITABLE (data->password_entry1)));
  gtk_window_destroy (GTK_WINDOW (window));
}


static void
cancel_button_pressed (GtkButton *widget,
                       GtkWidget *window)
{
  gtk_window_destroy (GTK_WINDOW (window));
}


static void
activate (GtkApplication* app,
          gpointer        user_data)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *cancel_button;

  window = gtk_application_window_new (app);
  gtk_window_set_title (GTK_WINDOW (window), "GCryFolder");
  gtk_window_set_icon_name(GTK_WINDOW (window), "encripted-folder");
  gtk_window_set_default_size (GTK_WINDOW (window), 300, 0);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
  gtk_window_set_child (GTK_WINDOW (window), vbox);
  
  gtk_widget_set_margin_top (vbox, 8);
  gtk_box_append (GTK_BOX (vbox), gtk_label_new(gettext ("Enter password for:")));
  gtk_box_append (GTK_BOX (vbox), gtk_label_new(data->filename));
  data->password_entry1 = compat_password_entry();
  g_object_set (data->password_entry1,
				"placeholder-text", gettext ("Password"),
                "activates-default", TRUE,
                NULL);
  g_signal_connect (data->password_entry1, "notify::text", G_CALLBACK (update_button), NULL);
  gtk_box_append (GTK_BOX (vbox), data->password_entry1);
  
  if (data->new_folder) {
    data->password_entry2 = compat_password_entry();
    g_object_set (data->password_entry2,
                  "placeholder-text", gettext ("Confirm password"),
                  "activates-default", TRUE,
                  NULL);
    g_signal_connect (data->password_entry2, "notify::text", G_CALLBACK (update_button), NULL);
    gtk_box_append (GTK_BOX (vbox), data->password_entry2);
  }
  
  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
  cancel_button = gtk_button_new_with_label(gettext ("Cancel"));
  g_signal_connect (cancel_button, "clicked", G_CALLBACK (cancel_button_pressed), window);
  gtk_box_append (GTK_BOX (hbox), cancel_button);
  data->ok_button = gtk_button_new_with_label(gettext ("OK"));
  gtk_widget_set_sensitive (data->ok_button, FALSE);
  gtk_widget_add_css_class (data->ok_button, "suggested-action");
  g_signal_connect (data->ok_button, "clicked", G_CALLBACK (ok_button_pressed), window);
  gtk_box_append (GTK_BOX (hbox), data->ok_button);
  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
  gtk_box_append (GTK_BOX (vbox), hbox);
  gtk_window_set_default_widget (GTK_WINDOW (window), data->ok_button);
  
  compat_widget_show (window);
}


static gint
command (GtkApplication         *app,
        GApplicationCommandLine *command_line,
        gpointer                 user_data)
{
  gint status;
  gint argc;
  gchar **argv;
  
  argv = g_application_command_line_get_arguments (command_line, &argc);
  status = parse_command_line(argc, argv);
  g_strfreev(argv);
  
  if (0 == status) activate (app, user_data);
  return status;
}


gint parse_command_line (gint    argc,
                         gchar **argv)
{
  gchar *cryfs;
  gchar *encfs;
  if (2 != argc) {
    g_print(gettext ("error: missing folder name\n"));
    g_print(gettext ("usage: %s /path/to/encrypted/folder\n"), argv[0]);
    return 1;
  }
  if (! g_file_test (argv[1], G_FILE_TEST_IS_DIR)) {
    g_print(gettext ("error: folder %s does not exist\n"), argv[1]);
    return 2;
  }
  data->filename = g_strdup(argv[1]);
  cryfs = g_strdup_printf("%s/cryfs.config", data->filename);
  encfs = g_strdup_printf("%s/.encfs6.xml", data->filename);
  data->new_folder = ! (g_file_test (cryfs, G_FILE_TEST_EXISTS) || g_file_test (encfs, G_FILE_TEST_EXISTS));
  g_free(encfs);
  g_free(cryfs);
  return 0;
}


gint
main (gint    argc,
      gchar **argv)
{
  setlocale (LC_ALL, "");
  bindtextdomain (PROJECT_NAME, LOCALE_DIR);
  bind_textdomain_codeset (PROJECT_NAME, "UTF-8");
  textdomain (PROJECT_NAME);
  
  GtkApplication *app;
  int status;
  
  data = g_new(AppData, 1);
  
  //app = gtk_application_new ("fr.endres.gcryfolder", G_APPLICATION_FLAGS_NONE);
  app = gtk_application_new ("fr.endres.gcryfolder", G_APPLICATION_HANDLES_COMMAND_LINE);
  //g_signal_connect (app, "activate", G_CALLBACK (activate), NULL);
  g_signal_connect (app, "command-line", G_CALLBACK (command), NULL);
  status = g_application_run (G_APPLICATION (app), argc, argv);
  
  g_object_unref (app);
  if (NULL != data->filename) g_free(data->filename);
  g_free(data);
  
  return status;
}
